/* Changes the GFF files from Pernille Nielson's browser to beds. */

#include "common.h"
#include "pernilleGff.h"
#include "easyGene.h"

void usage()
/* Explain usage and exit */
{
errAbort("usage:\n"
         "   pernilleGffToBed input.gff output.bed");
}

struct easyGene *convertGff(struct pernilleGff *gffList)
{
struct easyGene *list = NULL;
struct pernilleGff *per;
for (per = gffList; per != NULL; per = per->next)
    {
    struct easyGene *eg;
    AllocVar(eg);
    eg->chrom = "chr";
    eg->chromStart = per->startStop-1;
    eg->chromEnd = per->stopStart;
    eg->name = cloneString(per->annot);
    if (lastChar(eg->name) != '*')
	eg->genbank[0] = 'Y';
    else 
	eg->genbank[0] = 'N';
    stripChar(eg->name,'*');
    eg->score = 1000;
    if (sameString(per->feat,"ALT"))
	eg->score = 600;
    else if (sameString(per->feat,"ORF"))
	eg->score = 200;    
    eg->strand[0] = per->strand[0];
    eg->feat = cloneString(per->feat);
    eg->R = per->R;
    eg->frame = per->frame;
    eg->orf = cloneString(per->orf);
    eg->startCodon = cloneString(per->startCodon);
    stripChar(eg->startCodon,'#');
    eg->logOdds = per->logOdds;
    eg->descriptor = per->descriptor;
    eg->swissProt = cloneString(per->swissProt);
    slAddHead(&list,eg);
    }
slReverse(&list);
return list;
}

int main(int argc, char *argv[])
/* Just the main... */
{
struct pernilleGff *gffList;
struct easyGene *egList, *eg;
FILE *output;
if (argc < 3) 
    usage();
gffList = pernilleGffLoadAll(argv[1]);
egList = convertGff(gffList);
output = mustOpen(argv[2],"w");
for (eg = egList; eg != NULL; eg = eg->next)
    easyGeneTabOut(eg,output);
carefulClose(&output);
easyGeneFreeList(&egList);
pernilleGffFreeList(&gffList);
return 0;
}
